/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/**
 * @test TestHumongousShrinkHeap
 * @bug 8037925
 * @summary Verify that heap shrinking after GC of hard defragmented by humongous object heap
 * @author Andrey Zakharov /andrey.x.zakharov@oracle.com/
 * @author Dmitry Fazunenko /dmitry.fazunenko@oracle.com/
 * @library /testlibrary
 * @run main/othervm -XX:MinHeapFreeRatio=10 -XX:MaxHeapFreeRatio=50 -XX:+UseG1GC -XX:G1HeapRegionSize=1M -Xloggc:gc_10_50.log TestHumongousShrinkHeap
 */
import com.oracle.java.testlibrary.gc.MemoryUsagePrinter;
import com.oracle.java.testlibrary.gc.TestShrinkHeap;
import java.util.ArrayList;
import java.util.List;

public class TestHumongousShrinkHeap extends TestShrinkHeap {

    private static final ArrayList<ArrayList<byte[]>> garbage = new ArrayList<>();
    private static final int PAGE_SIZE = 1024 * 1024; // 1M 
    private static final int PAGES_NUM = 5;

    public TestHumongousShrinkHeap() {
        super();
    }

    public void eat() {
        int HumongousObjectSize = Math.round(.9f * PAGE_SIZE);
        System.out.println("Will allocate objects of size=" + 
                MemoryUsagePrinter.humanReadableByteCount(HumongousObjectSize, true));
        
        for (int i = 0; i < PAGES_NUM; i++) {
            ArrayList<byte[]> stuff = new ArrayList<>();
            eatList(stuff, 100, HumongousObjectSize);
            MemoryUsagePrinter.printMemoryUsage("eat #" + i);
            garbage.add(stuff);
        }
    }

    public void free() {
        //do not free last one list
        garbage.subList(0, garbage.size() - 1).clear();

        // do not free last one element from last list
        ArrayList stuff = garbage.get(garbage.size() - 1);
        stuff.subList(0, stuff.size() - 1).clear();
        System.gc();
    }

    public static void main(String[] args) {
        new TestHumongousShrinkHeap().test();
    }

    private static void eatList(List garbage, int count, int size) {
        for (int i = 0; i < count; i++) {
            garbage.add(new byte[size]);
        }
    }
}
